/**
 * jquery.gallery.js
 * http://www.codrops.com
 *
 * Copyright 2011, Pedro Botelho / Codrops
 * Free to use under the MIT license.
 *
 * Date: Mon Jan 30 2012
 */

(function( $, undefined ) {
	
	/*
	 * Gallery object.
	 */
	$.Gallery 				= function( options, element ) {
	
		this.$el	= $( element );
		this._init( options );
		
	};
	
	$.Gallery.defaults 		= {
		current		: 0,	// index of current item
		autoplay	: false,// slideshow on / off
		interval	: 4000  // time between transitions
    };
	
	$.Gallery.prototype 	= {
		_init 				: function( options ) {
			this.options 		= $.extend( true, {}, $.Gallery.defaults, options );
			
			// support for 3d / 2d transforms and transitions
			this.support3d		= Modernizr.csstransforms3d;
      // in application (if android 4.0) not support 'translateZ' I do not know why
      if (navigator.userAgent.toLowerCase().indexOf("android 4.0") != -1)
        this.support3d  = false;

			this.support2d		= Modernizr.csstransforms;
			this.supportTrans	= Modernizr.csstransitions;
			
			this.$wrapper		= this.$el.find('.dg-wrapper');
			this.$container		= $('.dg-container');

			this.$items			= this.$wrapper.children();
			this.itemsCount		= this.$items.length;
			
      this._zoom = window.innerHeight / 768;

			// minimum of 3 items
			if( this.itemsCount < 3 )
      {
				return false;
			}

      this.startMove = false;
      this.Distance = 0;
      this._translateX = 480;
      this._translateZ = 300;
			this._rotateY = 45;
      this._shift = 0;
      // переменные для определения скорости и направления скрола
      this.lineMotion = 'next'; // нужно назначить значение поумолчанию
      this.startX = 0;
      this._moveTo = 0;
      this.startTime = utils.getTime();
      //
      this.current		= this.options.current;
			
			this.isAnim			= false;
			this.isMove			= false;

			this.$items.css({
				'opacity'	: 0,
				'visibility': 'hidden'
			});

			this._validate();
			
			this._layout();
			
			// load the events
			this._loadEvents();
			
			// slideshow
			if( this.options.autoplay )
      {
				this._startSlideshow();
			}
		},

		_validate			: function() {
			if( this.options.current < 0 || this.options.current > this.itemsCount - 1 )
      {
				this.current = 0;
			}
		},

		_layout				: function() {
			// current, left and right items
			this._setItems();
			// current item is not changed
			// left and right one are rotated and translated
			var leftCSS, rightCSS, currentCSS;
			
			if( this.support3d && this.supportTrans )
      {
			
				leftCSS 	= {
					'-webkit-transform'	: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+this._rotateY+'deg)',
					'-moz-transform'	: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+this._rotateY+'deg)',
					'-o-transform'		: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+this._rotateY+'deg)',
					'-ms-transform'		: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+this._rotateY+'deg)',
					'transform'			: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+this._rotateY+'deg)'
				};
				
				rightCSS	= {
					'-webkit-transform'	: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+this._rotateY+'deg)',
					'-moz-transform'	: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+this._rotateY+'deg)',
					'-o-transform'		: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+this._rotateY+'deg)',
					'-ms-transform'		: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+this._rotateY+'deg)',
					'transform'			: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+this._rotateY+'deg)'
				};
				
				leftCSS.opacity		= 1;
				leftCSS.visibility	= 'visible';
				rightCSS.opacity	= 1;
				rightCSS.visibility	= 'visible';
			
			}
			else
      if( this.support2d && this.supportTrans )
      {
				leftCSS 	= {
					'-webkit-transform'	: 'translateX(-'+this._translateX+'px) scale(0.8)',
					'-moz-transform'	: 'translateX(-'+this._translateX+'px) scale(0.8)',
					'-o-transform'		: 'translateX(-'+this._translateX+'px) scale(0.8)',
					'-ms-transform'		: 'translateX(-'+this._translateX+'px) scale(0.8)',
					'transform'			: 'translateX(-'+this._translateX+'px) scale(0.8)'
				};
				
				rightCSS	= {
					'-webkit-transform'	: 'translateX('+this._translateX+'px) scale(0.8)',
					'-moz-transform'	: 'translateX('+this._translateX+'px) scale(0.8)',
					'-o-transform'		: 'translateX('+this._translateX+'px) scale(0.8)',
					'-ms-transform'		: 'translateX('+this._translateX+'px) scale(0.8)',
					'transform'			: 'translateX('+this._translateX+'px) scale(0.8)'
				};
				
				currentCSS	= {
					'z-index'			: 999
				};
				
				leftCSS.opacity		= 1;
				leftCSS.visibility	= 'visible';
				rightCSS.opacity	= 1;
				rightCSS.visibility	= 'visible';
			}
			
			this.$leftItm.css( leftCSS || {} );
			this.$rightItm.css( rightCSS || {} );
			
			this.$currentItm.css( currentCSS || {} ).css({
        'z-index' : 999,
				'opacity'	: 1,
				'visibility': 'visible'
			}).addClass('dg-center');
		},

		_setItems			: function() {
			this.$items.removeClass('dg-center');
			this.$currentItm	= this.$items.eq( this.current );
			this.$leftItm		= ( this.current === 0 ) ? this.$items.eq( this.itemsCount - 1 ) : this.$items.eq( this.current - 1 );
			this.$rightItm		= ( this.current === this.itemsCount - 1 ) ? this.$items.eq( 0 ) : this.$items.eq( this.current + 1 );
			
			if( !this.support3d && this.support2d && this.supportTrans )
      {
				this.$items.css( {
          'z-index': 1,
          'opacity': 0
        });
				this.$leftItm.css({
          'z-index': 899,
          'opacity': 1
        });
				this.$rightItm.css({
          'z-index': 899,
          'opacity': 1
        });
				this.$currentItm.css({
          'z-index': 999,
          'opacity': 1
        });
			}
			// next & previous items
			if( this.itemsCount > 3 )
      {
				// next item
				this.$nextItm		= ( this.$rightItm.index() === this.itemsCount - 1 ) ? this.$items.eq( 0 ) : this.$rightItm.next();
				this.$nextItm.css( this._getCoordinates('outright') );
				// previous item
				this.$prevItm		= ( this.$leftItm.index() === 0 ) ? this.$items.eq( this.itemsCount - 1 ) : this.$leftItm.prev();
				this.$prevItm.css( this._getCoordinates('outleft') );
			}
		},

		_loadEvents			: function() {
      $(window).resize(function(){
        _self._zoom = window.innerHeight / 768;
      });
			var _self	= this;

			this.$wrapper.on( 'webkitTransitionEnd.gallery transitionend.gallery OTransitionEnd.gallery', function( event ) {
				_self.$currentItm.addClass('dg-center');
				_self.$items.removeClass('dg-transition'); //нужно будет удалить вообще
        _self.$items.css({
            '-webkit-transition'  : '',
            '-moz-transition'     : '',
            '-o-transition'       : '',
            '-ms-transition'      : '',
            'transition'          : ''
          });

        _self.isAnim  = false;
        _self.isMove  = false;
			});

      this.$container.on('click', function(e) {
        if( _self.isMove )
          return false;

        var tubPosition = e.pageX / _self._zoom;
        var leftBoundaryPoint = _self.$wrapper[0].offsetLeft;
        var rightBoundaryPoint = _self.$wrapper[0].offsetLeft + _self._translateX;
        var offset = _self._translateX / _self._zoom / 2;

        if( (_self.support3d && _self.supportTrans) )
        {
          if ((tubPosition < leftBoundaryPoint) && (tubPosition > (leftBoundaryPoint - offset)))
          {
            try{
              window.location.href =  _self.$leftItm[0].href;
            } catch(error) {
              alert(error.name+' - '+ error.message);
            }
          }
          else
          if((tubPosition > rightBoundaryPoint) && (tubPosition < (rightBoundaryPoint + offset)))
          {
            window.location.href =  _self.$rightItm[0].href;
            //_self.$rightItm[0].click();
          }
        }
      });
      this.$container.bind('movestart', function(e) {
          //_self.options.autoplay	= false;
          if( _self.supportTrans && _self.isAnim )
            return false;

          _self.startMove = true;
          _self.Distance = 0;
          _self._shift = 0;
          if( _self.options.autoplay ) {
            clearTimeout( _self.slideshow );
            _self.options.autoplay	= false;
          }
        _self.isMove = true;

      }).bind('move', function(e) {
          if( _self.supportTrans && _self.isAnim )
            return false;

          _self.lineMotion = (e.deltaX < 0) ? 'next' : 'prev';
          var timestamp  = utils.getTime();
          if ( timestamp - _self.startTime > 300 )
          {
            _self.startTime = timestamp;
            _self.startX = e.pageX/_self._zoom;
          }
          _self.Distance = e.startX/_self._zoom - e.pageX/_self._zoom - _self._shift;

          if( !(_self.support3d && _self.supportTrans) )
            return false;

          if (_self.Distance > _self._translateX)
          {
            _self.Distance = 0;
            _self._shift += _self._translateX;
            _self.current	= _self.$rightItm.index();
            _self._setItems();
          }
          else
          if((-1 * _self.Distance) > _self._translateX)
          {
            _self.Distance = 0;
            _self._shift -= _self._translateX;
            _self.current	= _self.$leftItm.index();
            _self._setItems();
          }
          _self._displacement(_self.Distance);
      }).bind('moveend', function(e) {
          _self.startMove = false;

          if( !(_self.support3d && _self.supportTrans) )
            return _self._navigate(_self.lineMotion, 5);

          var duration = utils.getTime() - _self.startTime,
            speed = 0;

          // start momentum animation if needed
          if ( duration < 300 )
            speed = Math.abs(e.pageX/_self._zoom - _self.startX) / duration;

          if( speed < 0.8 )
          {
            _self._endDisplacement();
            //$("#trace").html("v7 speed="+speed + " duration=" + duration + " Distance=" + _self.Distance);
          }
          else
          {
            //_self._moveTo = (_self.lineMotion == 'next')? _self.Distance + duration * speed : _self.Distance - duration * speed;
            _self._moveTo = (_self.lineMotion == 'next')?  _self._translateX - _self.Distance : -_self._translateX - _self.Distance;

            //$("#trace").html("v7 speed="+speed + " duration=" + duration + " Distance=" + _self.Distance + " _moveTo=" + _self._moveTo);
            _self._startDisplacement();
          }
        });
		},

    _startDisplacement: function () {
      var _self	= this;
      if (_self.startMove)
      {
        return false;
        clearTimeout(_self.Displacement);
      }

      _self.Distance += _self._moveTo/2.7;
      _self._moveTo -= _self._moveTo/2.7;

      if (_self.Distance > _self._translateX)
      {
        //_self.Distance = 0;
        _self.Distance -= _self._translateX;
        //_self._shift += _self._translateX;
        _self.current	= _self.$rightItm.index();
        _self._setItems();
      }
      else
      if((-1 * _self.Distance) > _self._translateX)
      {
        //_self.Distance = 0;
        _self.Distance += _self._translateX;
        _self.current	= _self.$leftItm.index();
        _self._setItems();
      }
      this._displacement(this.Distance);
      this.Displacement	= setTimeout( function() {
        if( !(_self._moveTo < 1 && _self._moveTo > -1 ) )
        {
          _self._startDisplacement();
        }
        else
        {
          clearTimeout(_self.Displacement);
          _self._endDisplacement();
        }
      }, 1000/60 );
    },

    _endDisplacement: function () {
      var _self	= this,
        border_passage = _self._translateX / 2,
        lineMotion = 'next',
        speed = 5;

      if( _self.Distance < border_passage && _self.Distance > (-1 * border_passage))
      {
        lineMotion = 'stay';
      }
      else
      {
        if(_self.Distance < 0)
        {
          lineMotion = 'prev';
          _self.Distance *= -1;
        }
        else
        {
          lineMotion = 'next';
        }
      }
      _self.Distance = (_self.Distance < border_passage)? _self.Distance : _self._translateX - _self.Distance;
      speed = Math.round(_self.Distance / 100);
      speed = (speed > 0) ? speed : -1 * speed;
      speed += 1;
      _self._navigate(lineMotion, speed);

      _self.options.autoplay = true;
      _self._startSlideshow();
    },

		_getCoordinates		: function( position ) {
			if( this.support3d && this.supportTrans )
      {
				switch( position ) {
					case 'outleft':
						return {
							'-webkit-transform'	: 'translate3d(-'+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translate3d(-'+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'-o-transform'		: 'translate3d(-'+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translate3d(-'+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'transform'			: 'translate3d(-'+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							/*'-webkit-transform'	: 'translateX(-'+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translateX(-'+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'-o-transform'		: 'translateX(-'+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translateX(-'+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',
							'transform'			: 'translateX(-'+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY('+(this._rotateY)+'deg)',*/
							'opacity'			: 0,
							'visibility'		: 'hidden'
						};
						break;
					case 'outright':
						return {
							'-webkit-transform'	: 'translate3d('+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translate3d('+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-o-transform'		: 'translate3d('+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translate3d('+(100+this._translateX)+'px, 0px, -'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'transform'			: 'translate3d('+(100+this._translateX)+'pxpx, 0px, -'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							/*'-webkit-transform'	: 'translateX('+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translateX('+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-o-transform'		: 'translateX('+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translateX('+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',
							'transform'			: 'translateX('+(100+this._translateX)+'px) translateZ(-'+(100+this._translateZ)+'px) rotateY(-'+(this._rotateY)+'deg)',*/
							'opacity'			: 0,
							'visibility'		: 'hidden'
						};
						break;
					case 'left':
            return {
              '-webkit-transform'	: 'translate3d(-'+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
              '-moz-transform'	: 'translate3d(-'+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
              '-o-transform'		: 'translate3d(-'+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
              '-ms-transform'		: 'translate3d(-'+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
              'transform'			: 'translate3d(-'+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
               /*'-webkit-transform'	: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
							'-o-transform'		: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',
							'transform'			: 'translateX(-'+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY('+(this._rotateY)+'deg)',*/
							'opacity'			: 1,
							'visibility'		: 'visible'
						};

						break;
					case 'right':
						return {
							'-webkit-transform'	: 'translate3d('+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translate3d('+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-o-transform'		: 'translate3d('+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translate3d('+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'transform'			: 'translate3d('+this._translateX+'px, 0px, -'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							/*'-webkit-transform'	: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-moz-transform'	: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-o-transform'		: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'-ms-transform'		: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',
							'transform'			: 'translateX('+this._translateX+'px) translateZ(-'+this._translateZ+'px) rotateY(-'+(this._rotateY)+'deg)',*/
							'opacity'			: 1,
							'visibility'		: 'visible'
						};
						break;
					case 'center':
						return {
							'-webkit-transform'	: 'translate3d(0px, 0px, 0px) rotateY(0deg)',
							'-moz-transform'	  : 'translate3d(0px, 0px, 0px) rotateY(0deg)',
							'-o-transform'		  : 'translate3d(0px, 0px, 0px) rotateY(0deg)',
							'-ms-transform'		  : 'translate3d(0px, 0px, 0px) rotateY(0deg)',
							'transform'			    : 'translate3d(0px, 0px, 0px) rotateY(0deg)',
							/*'-webkit-transform'	: 'translateX(0px) translateZ(0px) rotateY(0deg)',
							'-moz-transform'	: 'translateX(0px) translateZ(0px) rotateY(0deg)',
							'-o-transform'		: 'translateX(0px) translateZ(0px) rotateY(0deg)',
							'-ms-transform'		: 'translateX(0px) translateZ(0px) rotateY(0deg)',
							'transform'			: 'translateX(0px) translateZ(0px) rotateY(0deg)',*/
							'opacity'			: 1,
							'visibility'		: 'visible'
						};
						break;
				};
			}
			else
      if( this.support2d && this.supportTrans )
      {
				switch( position ) {
					case 'outleft':
						return {
							'-webkit-transform'	: 'translateX(-'+(100+this._translateX)+'px) scale(0.6)',
							'-moz-transform'	: 'translateX(-'+(100+this._translateX)+'px) scale(0.6)',
							'-o-transform'		: 'translateX(-'+(100+this._translateX)+'px) scale(0.6)',
							'-ms-transform'		: 'translateX(-'+(100+this._translateX)+'px) scale(0.6)',
							'transform'			  : 'translateX(-'+(100+this._translateX)+'px) scale(0.6)',
							'opacity'			    : 0,
							'visibility'		  : 'visible'
						};
						break;
					case 'outright':
						return {
							'-webkit-transform'	: 'translateX('+(100+this._translateX)+'px) scale(0.6)',
							'-moz-transform'	: 'translateX('+(100+this._translateX)+'px) scale(0.6)',
							'-o-transform'		: 'translateX('+(100+this._translateX)+'px) scale(0.6)',
							'-ms-transform'		: 'translateX('+(100+this._translateX)+'px) scale(0.6)',
							'transform'			  : 'translateX('+(100+this._translateX)+'px) scale(0.6)',
							'opacity'			    : 0,
							'visibility'		  : 'visible'
						};
						break;
					case 'left':
						return {
							'-webkit-transform'	: 'translateX(-'+this._translateX+'px) scale(0.8)',
							'-moz-transform'	: 'translateX(-'+this._translateX+'px) scale(0.8)',
							'-o-transform'		: 'translateX(-'+this._translateX+'px) scale(0.8)',
							'-ms-transform'		: 'translateX(-'+this._translateX+'px) scale(0.8)',
							'transform'			  : 'translateX(-'+this._translateX+'px) scale(0.8)',
							'opacity'			    : 1,
							'visibility'		  : 'visible'
						};
						break;
					case 'right':
						return {
							'-webkit-transform'	: 'translateX('+this._translateX+'px) scale(0.8)',
							'-moz-transform'	: 'translateX('+this._translateX+'px) scale(0.8)',
							'-o-transform'		: 'translateX('+this._translateX+'px) scale(0.8)',
							'-ms-transform'		: 'translateX('+this._translateX+'px) scale(0.8)',
							'transform'			  : 'translateX('+this._translateX+'px) scale(0.8)',
							'opacity'			    : 1,
							'visibility'		  : 'visible'
						};
						break;
					case 'center':
						return {
							'-webkit-transform'	: 'translateX(0px) scale(1)',
							'-moz-transform'	: 'translateX(0px) scale(1)',
							'-o-transform'		: 'translateX(0px) scale(1)',
							'-ms-transform'		: 'translateX(0px) scale(1)',
							'transform'			  : 'translateX(0px) scale(1)',
							'opacity'			    : 1,
							'visibility'		  : 'visible'
						};
						break;
				};
			}
			else
      {
				switch( position ) {
					case 'outleft'	: 
					case 'outright'	: 
					case 'left'		: 
					case 'right'	:
						return {
							'opacity'			: 0,
							'visibility'		: 'hidden'
						};
						break;
					case 'center'	:
						return {
							'opacity'			: 1,
							'visibility'		: 'visible'
						};
						break;
				};
			}
		},
    _getTransition  :function(spead){
      if( this.support3d && this.supportTrans ) {
        return {
          '-webkit-transition'  : 'all 0.'+spead+'s ease-in-out',
          '-moz-transition'     : 'all 0.'+spead+'s ease-in-out',
          '-o-transition'       : 'all 0.'+spead+'s ease-in-out',
          '-ms-transition'      : 'all 0.'+spead+'s ease-in-out',
          'transition'          : 'all 0.'+spead+'s ease-in-out'
        };
      }
      else
      if( this.support2d && this.supportTrans )
      {
        return {
          '-webkit-transition'  : '0.'+spead+'s ease-in-out',
          '-moz-transition'     : '0.'+spead+'s ease-in-out',
          '-o-transition'       : '0.'+spead+'s ease-in-out',
          '-ms-transition'      : '0.'+spead+'s ease-in-out',
          'transition'          : '0.'+spead+'s ease-in-out',
          '-webkit-transition-property' : '-webkit-transform, opacity',
          '-moz-transition-property'    : 'transform, opacity',
          '-o-transition-property'      : 'transform, opacity',
          '-ms-transition-property'     : 'transform, opacity',
          'transition-property'         : 'transform, opacity'
        };
      }
    },
		_navigate			: function( dir, spead ) {
			if( this.supportTrans && this.isAnim )
				return false;
      /* Иногда может прийти spead = 0. А при таком значении не произойдет событие TransitionEnd.gallery
      в котором происходит возврат к начальному состоянию (isAnim = false) и мы не можем двигать карусель */
      if(spead !== 0)
        this.isAnim	= true;
			
			switch( dir ) {
				case 'next' :
					this.current	= this.$rightItm.index();
					// current item moves left
					this.$currentItm.css( this._getTransition(spead) ).css( this._getCoordinates('left') );
					// right item moves to the center
					this.$rightItm.css( this._getTransition(spead) ).css( this._getCoordinates('center') );
					// next item moves to the right
					if( this.$nextItm ) {
						// left item moves out
						this.$leftItm.css( this._getTransition(spead) ).css( this._getCoordinates('outleft') );
						this.$nextItm.css( this._getTransition(spead) ).css( this._getCoordinates('right') );
					}
					else {
						// left item moves right
						this.$leftItm.css( this._getTransition(spead) ).css( this._getCoordinates('right') );
					}
					break;
					
				case 'prev' :
					this.current	= this.$leftItm.index();
					// current item moves right
					this.$currentItm.css( this._getTransition(spead) ).css( this._getCoordinates('right') );
					// left item moves to the center
					this.$leftItm.css( this._getTransition(spead) ).css( this._getCoordinates('center') );
					// prev item moves to the left
					if( this.$prevItm ) {
						// right item moves out
						this.$rightItm.css( this._getTransition(spead) ).css( this._getCoordinates('outright') );
						this.$prevItm.css( this._getTransition(spead) ).css( this._getCoordinates('left') );
					}
					else {
						// right item moves left
						this.$rightItm.css( this._getTransition() ).css( this._getCoordinates('left') );
					}
					break;

				case 'stay' :
					this.$currentItm.css( this._getTransition(spead) ).css( this._getCoordinates('center') );
					this.$leftItm.css( this._getTransition(spead) ).css( this._getCoordinates('left') );
					this.$rightItm.css( this._getTransition(spead) ).css( this._getCoordinates('right') );
					this.$prevItm.css( this._getTransition(spead) ).css( this._getCoordinates('outleft') );
          this.$nextItm.css( this._getTransition(spead) ).css( this._getCoordinates('outright') );
          return false;
					break;
			};

			this._setItems();
			
			if( !this.supportTrans )
				this.$currentItm.addClass('dg-center');
			
		},

		_displacement			: function( dir ) {
			if( dir > 0)
      {
        if (dir > this._translateX)
          return false;
        var rttY = this._rotateY*(dir/this._translateX);
        var trnZ = this._translateZ*(dir/this._translateX);
        var opc = 1*(dir/this._translateX);

        var outL = 100*(dir/this._translateX);

				this.current	= this.$rightItm.index();
					// current item moves left
        if( this.support3d && this.supportTrans )
        {
					this.$currentItm.css(  {
            '-webkit-transform'	: 'translateX(-'+ dir +'px) translateZ(-'+trnZ+'px) rotateY('+rttY+'deg)',
            '-moz-transform'	  : 'translateX(-'+ dir +'px) translateZ(-'+trnZ+'px) rotateY('+rttY+'deg)',
            '-o-transform'		  : 'translateX(-'+ dir +'px) translateZ(-'+trnZ+'px) rotateY('+rttY+'deg)',
            '-ms-transform'		  : 'translateX(-'+ dir +'px) translateZ(-'+trnZ+'px) rotateY('+rttY+'deg)',
            'transform'			    : 'translateX(-'+ dir +'px) translateZ(-'+trnZ+'px) rotateY('+rttY+'deg)',
            'opacity'			      : 1,
            'z-index'           : 999,
            'visibility'		    : 'visible'
          } );

					// right item moves to the center
					this.$rightItm.css( {
            '-webkit-transform'	: 'translateX('+ (this._translateX - dir) +'px) translateZ(-'+(this._translateZ - trnZ)+'px) rotateY(-'+(45-rttY)+'deg)',
            '-moz-transform'	  : 'translateX('+ (this._translateX - dir) +'px) translateZ(-'+(this._translateZ - trnZ)+'px) rotateY(-'+(45-rttY)+'deg)',
            '-o-transform'		  : 'translateX('+ (this._translateX - dir) +'px) translateZ(-'+(this._translateZ - trnZ)+'px) rotateY(-'+(45-rttY)+'deg)',
            '-ms-transform'		  : 'translateX('+ (this._translateX - dir) +'px) translateZ(-'+(this._translateZ - trnZ)+'px) rotateY(-'+(45-rttY)+'deg)',
            'transform'			    : 'translateX('+ (this._translateX - dir) +'px) translateZ(-'+(this._translateZ - trnZ)+'px) rotateY(-'+(45-rttY)+'deg)',
            'opacity'			      : 1,
            'z-index'           : 1,
            'visibility'		    : 'visible'
          } );
					// next item moves to the right
					if( this.$nextItm )
          {
						// left item moves out
						this.$leftItm.css( {
              '-webkit-transform'	: 'translateX(-' + (this._translateX + outL) + 'px) translateZ(-' + (this._translateZ + outL) + 'px) rotateY(45deg)',
              '-moz-transform'	  : 'translateX(-' + (this._translateX + outL) + 'px) translateZ(-' + (this._translateZ + outL) + 'px) rotateY(45deg)',
              '-o-transform'		  : 'translateX(-' + (this._translateX + outL) + 'px) translateZ(-' + (this._translateZ + outL) + 'px) rotateY(45deg)',
              '-ms-transform'		  : 'translateX(-' + (this._translateX + outL) + 'px) translateZ(-' + (this._translateZ + outL) + 'px) rotateY(45deg)',
              'transform'			    : 'translateX(-' + (this._translateX + outL) + 'px) translateZ(-' + (this._translateZ + outL) + 'px) rotateY(45deg)',
              'opacity'			      : (1 - opc),
              'visibility'		    : 'visible'
            } );
						this.$nextItm.css( {
              '-webkit-transform'	: 'translateX(' + (100+this._translateX - outL) + 'px) translateZ(-' + (100+this._translateZ - outL) + 'px) rotateY(-45deg)',
              '-moz-transform'	  : 'translateX(' + (100+this._translateX - outL) + 'px) translateZ(-' + (100+this._translateZ - outL) + 'px) rotateY(-45deg)',
              '-o-transform'		  : 'translateX(' + (100+this._translateX - outL) + 'px) translateZ(-' + (100+this._translateZ - outL) + 'px) rotateY(-45deg)',
              '-ms-transform'		  : 'translateX(' + (100+this._translateX - outL) + 'px) translateZ(-' + (100+this._translateZ - outL) + 'px) rotateY(-45deg)',
              'transform'			    : 'translateX(' + (100+this._translateX - outL) + 'px) translateZ(-' + (100+this._translateZ - outL) + 'px) rotateY(-45deg)',
              'opacity'			      : opc,
              'visibility'		    : 'visible'
            } );
					}
					else
          {
						// left item moves right
						// если у нас всего три картинки то будет это событие
						//this.$leftItm.css( this._getCoordinates('right') );
            alert('Have We only three banners?');
						this.$leftItm.css( {
              '-webkit-transform'	: 'translateX(350px) translateZ(-200px) rotateY(-45deg)',
              '-moz-transform'	  : 'translateX(350px) translateZ(-200px) rotateY(-45deg)',
              '-o-transform'		  : 'translateX(350px) translateZ(-200px) rotateY(-45deg)',
              '-ms-transform'		  : 'translateX(350px) translateZ(-200px) rotateY(-45deg)',
              'transform'			    : 'translateX(350px) translateZ(-200px) rotateY(-45deg)',
              'opacity'			      : 1,
              'visibility'		    : 'visible'
            } );
					}
        }
        else
        if(this.support2d && this.supportTrans)
        {
          var _scale =  0.2*(dir/this._translateX);
          // current item moves left
          this.$currentItm.css(  {
            '-webkit-transform'	: 'scale('+(1-_scale)+') translateX(-'+ dir +'px)',
            '-moz-transform'	  : 'scale('+(1-_scale)+') translateX(-'+ dir +'px)',
            '-o-transform'		  : 'scale('+(1-_scale)+') translateX(-'+ dir +'px)',
            '-ms-transform'		  : 'scale('+(1-_scale)+') translateX(-'+ dir +'px)',
            'transform'			    : 'scale('+(1-_scale)+') translateX(-'+ dir +'px)',
            'opacity'			      : 1,
            'visibility'		    : 'visible'
          } );
          // right item moves to the center
          this.$rightItm.css(  {
            '-webkit-transform'	: 'scale('+(0.8+_scale)+') translateX('+ (100+this._translateX - dir) +'px)',
            '-moz-transform'	  : 'scale('+(0.8+_scale)+') translateX('+ (100+this._translateX - dir) +'px)',
            '-o-transform'		  : 'scale('+(0.8+_scale)+') translateX('+ (100+this._translateX - dir) +'px)',
            '-ms-transform'		  : 'scale('+(0.8+_scale)+') translateX('+ (100+this._translateX - dir) +'px)',
            'transform'			    : 'scale('+(0.8+_scale)+') translateX('+ (100+this._translateX - dir) +'px)',
            'opacity'			      : 1,
            'z-index'           : 999,
            'visibility'		    : 'visible'
          } );

          this.$leftItm.css( {
            '-webkit-transform'	: 'scale('+(0.8-_scale)+') translateX(-' + (this._translateX + outL) + 'px)',
            '-moz-transform'	  : 'scale('+(0.8-_scale)+') translateX(-' + (this._translateX + outL) + 'px)',
            '-o-transform'		  : 'scale('+(0.8-_scale)+') translateX(-' + (this._translateX + outL) + 'px)',
            '-ms-transform'		  : 'scale('+(0.8-_scale)+') translateX(-' + (this._translateX + outL) + 'px)',
            'transform'			    : 'scale('+(0.8-_scale)+') translateX(-' + (this._translateX + outL) + 'px)',
            'opacity'			      : (1 - opc),
            'visibility'		    : 'visible'
          } );
          this.$nextItm.css( {
            '-webkit-transform'	: 'scale('+(0.6+_scale)+') translateX(' + (100+this._translateX - outL) + 'px)',
            '-moz-transform'	  : 'scale('+(0.6+_scale)+') translateX(' + (100+this._translateX - outL) + 'px)',
            '-o-transform'		  : 'scale('+(0.6+_scale)+') translateX(' + (100+this._translateX - outL) + 'px)',
            '-ms-transform'		  : 'scale('+(0.6+_scale)+') translateX(' + (100+this._translateX - outL) + 'px)',
            'transform'			    : 'scale('+(0.6+_scale)+') translateX(' + (100+this._translateX - outL) + 'px)',
            'opacity'			      : opc,
            'visibility'		    : 'visible'
          } );
        }
      }
      else
      {
        dir *= -1;
        if (dir > this._translateX)
          return false;

        var rttY = this._rotateY*(dir/this._translateX);
        var trnZ = this._translateZ*(dir/this._translateX);
        var opc = 1*(dir/this._translateX);

        var outL = 100*(dir/this._translateX);

        this.current	= this.$leftItm.index();
        if( this.support3d && this.supportTrans )
        {
  				// current item moves right
	  			this.$currentItm.css( {
            '-webkit-transform'	: 'translateX('+dir+'px) translateZ(-'+trnZ+'px) rotateY(-'+rttY+'deg)',
            '-moz-transform'	: 'translateX('+dir+'px) translateZ(-'+trnZ+'px) rotateY(-'+rttY+'deg)',
            '-o-transform'		: 'translateX('+dir+'px) translateZ(-'+trnZ+'px) rotateY(-'+rttY+'deg)',
            '-ms-transform'		: 'translateX('+dir+'px) translateZ(-'+trnZ+'px) rotateY(-'+rttY+'deg)',
            'transform'			: 'translateX('+dir+'px) translateZ(-'+trnZ+'px) rotateY(-'+rttY+'deg)',
            'opacity'			: 1,
            'visibility'		: 'visible'
          } );
				  // left item moves to the center
				  this.$leftItm.css( {
            '-webkit-transform'	: 'translateX(-'+(this._translateX-dir)+'px) translateZ(-'+(this._translateZ-trnZ)+'px) rotateY('+(45-rttY)+'deg)',
            '-moz-transform'	: 'translateX(-'+(this._translateX-dir)+'px) translateZ(-'+(this._translateZ-trnZ)+'px) rotateY('+(45-rttY)+'deg)',
            '-o-transform'		: 'translateX(-'+(this._translateX-dir)+'px) translateZ(-'+(this._translateZ-trnZ)+'px) rotateY('+(45-rttY)+'deg)',
            '-ms-transform'		: 'translateX(-'+(this._translateX-dir)+'px) translateZ(-'+(this._translateZ-trnZ)+'px) rotateY('+(45-rttY)+'deg)',
            'transform'			: 'translateX(-'+(this._translateX-dir)+'px) translateZ(-'+(this._translateZ-trnZ)+'px) rotateY('+(45-rttY)+'deg)',
            'opacity'			: 1,
            'visibility'		: 'visible'
          } );
	  			// prev item moves to the left
		  		if( this.$prevItm ) {
			  		// right item moves out
				  	this.$rightItm.css( {
              '-webkit-transform'	: 'translateX('+(this._translateX+outL)+'px) translateZ(-'+(this._translateZ+outL)+'px) rotateY(-45deg)',
              '-moz-transform'	: 'translateX('+(this._translateX+outL)+'px) translateZ(-'+(this._translateZ+outL)+'px) rotateY(-45deg)',
              '-o-transform'		: 'translateX('+(this._translateX+outL)+'px) translateZ(-'+(this._translateZ+outL)+'px) rotateY(-45deg)',
              '-ms-transform'		: 'translateX('+(this._translateX+outL)+'px) translateZ(-'+(this._translateZ+outL)+'px) rotateY(-45deg)',
              'transform'			: 'translateX('+(this._translateX+outL)+'px) translateZ(-'+(this._translateZ+outL)+'px) rotateY(-45deg)',
              'opacity'			: (1-opc),
              'visibility'		: 'visible'
            } );
  					this.$prevItm.css( {
              '-webkit-transform'	: 'translateX(-'+(100+this._translateX-outL)+'px) translateZ(-'+(100+this._translateZ-outL)+'px) rotateY(45deg)',
              '-moz-transform'	: 'translateX(-'+(100+this._translateX-outL)+'px) translateZ(-'+(100+this._translateZ-outL)+'px) rotateY(45deg)',
              '-o-transform'		: 'translateX(-'+(100+this._translateX-outL)+'px) translateZ(-'+(100+this._translateZ-outL)+'px) rotateY(45deg)',
              '-ms-transform'		: 'translateX(-'+(100+this._translateX-outL)+'px) translateZ(-'+(100+this._translateZ-outL)+'px) rotateY(45deg)',
              'transform'			: 'translateX(-'+(100+this._translateX-outL)+'px) translateZ(-'+(100+this._translateZ-outL)+'px) rotateY(45deg)',
              'opacity'			: opc,
              'visibility'		: 'visible'
            } );
  				}
	  			else
          {
			  		// right item moves left
				  	this.$rightItm.css( this._getCoordinates('left') );
  				}/**/
        }
        else
        if(this.support2d && this.supportTrans)
        {
          var _scale =  0.2*(dir/this._translateX);
          this.$currentItm.css(  {
            '-webkit-transform'	: 'scale('+(1-_scale)+') translateX('+ dir +'px)',
            '-moz-transform'	: 'scale('+(1-_scale)+') translateX('+ dir +'px)',
            '-o-transform'		: 'scale('+(1-_scale)+') translateX('+ dir +'px)',
            '-ms-transform'		: 'scale('+(1-_scale)+') translateX('+ dir +'px)',
            'transform'			: 'scale('+(1-_scale)+') translateX('+ dir +'px)',
            'opacity'			: 1,
            'visibility'		: 'visible'
          } );

          this.$leftItm.css(  {
            '-webkit-transform'	: 'scale('+(0.8+_scale)+') translateX(-'+ (100+this._translateX - dir) +'px)',
            '-moz-transform'	: 'scale('+(0.8+_scale)+') translateX(-'+ (100+this._translateX - dir) +'px)',
            '-o-transform'		: 'scale('+(0.8+_scale)+') translateX(-'+ (100+this._translateX - dir) +'px)',
            '-ms-transform'		: 'scale('+(0.8+_scale)+') translateX(-'+ (100+this._translateX - dir) +'px)',
            'transform'			: 'scale('+(0.8+_scale)+') translateX(-'+ (100+this._translateX - dir) +'px)',
            'opacity'			: 1,
            'visibility'		: 'visible'
          } );

          this.$rightItm.css( {
            '-webkit-transform'	: 'scale('+(0.8-_scale)+') translateX(' + (this._translateX - outL) + 'px)',
            '-moz-transform'	: 'scale('+(0.8-_scale)+') translateX(' + (this._translateX - outL) + 'px)',
            '-o-transform'		: 'scale('+(0.8-_scale)+') translateX(' + (this._translateX - outL) + 'px)',
            '-ms-transform'		: 'scale('+(0.8-_scale)+') translateX(' + (this._translateX - outL) + 'px)',
            'transform'			: 'scale('+(0.8-_scale)+') translateX(' + (this._translateX - outL) + 'px)',
            'opacity'			: (1 - opc),
            'visibility'		: 'visible'
          } );

          this.$prevItm.css( {
            '-webkit-transform'	: 'scale('+(0.6+_scale)+') translateX(-' + (100+this._translateX - outL) + 'px)',
            '-moz-transform'	  : 'scale('+(0.6+_scale)+') translateX(-' + (100+this._translateX - outL) + 'px)',
            '-o-transform'		  : 'scale('+(0.6+_scale)+') translateX(-' + (100+this._translateX - outL) + 'px)',
            '-ms-transform'		  : 'scale('+(0.6+_scale)+') translateX(-' + (100+this._translateX - outL) + 'px)',
            'transform'			    : 'scale('+(0.6+_scale)+') translateX(-' + (100+this._translateX - outL) + 'px)',
            'opacity'			      : opc,
            'visibility'		    : 'visible'
          } );

        }
			};
		},

		_startSlideshow		: function() {
			var _self	= this;
			this.slideshow	= setTimeout( function() {
				_self._navigate( 'next', 5 );
				if( _self.options.autoplay )
        {
					_self._startSlideshow();
				}
			}, this.options.interval );
		},

		destroy				: function() {
			this.$wrapper.off('.gallery');
		}
	};
	
	var logError 			= function( message ) {
		if ( this.console )
    {
			console.error( message );
		}
	};
	
	$.fn.gallery			= function( options ) {
		if ( typeof options === 'string' )
    {
			var args = Array.prototype.slice.call( arguments, 1 );
			this.each(function() {
				var instance = $.data( this, 'gallery' );
				if ( !instance )
        {
					logError( "cannot call methods on gallery prior to initialization; " +
					"attempted to call method '" + options + "'" );
					return;
				}
				if ( !$.isFunction( instance[options] ) || options.charAt(0) === "_" )
        {
					logError( "no such method '" + options + "' for gallery instance" );
					return;
				}
				instance[ options ].apply( instance, args );
			});
		}
		else
    {
			this.each(function() {
				var instance = $.data( this, 'gallery' );
				if ( !instance ) {
					$.data( this, 'gallery', new $.Gallery( options, this ) );
				}
			});
		}
		return this;
	};

  /*
  * взял утилиту из библиотеки IScroll
  *
  */
  var utils = (function () {
    var me = {};

    me.getTime = Date.now || function getTime () { return new Date().getTime(); };
    me.getTouch = function (_evt) {

      var x = 0;//whw.w;
      var y = 0;//whw.h;
      var flag = true;
      if (typeof _evt.touches != 'undefined')
      {
        if (_evt.touches.length > 0)
        {
          if (typeof _evt.touches[0].clientX != 'undefined' && typeof _evt.touches[0].clientY != 'undefined')
          {
            x = _evt.touches[0].clientX;
            y = _evt.touches[0].clientY;
          }
          else
          {
            x = _evt.touches[0].pageX;
            y = _evt.touches[0].pageY;
          }
          flag = false;
        }
      }
      if (flag)
      {
        if (typeof _evt.clientX != 'undefined' && typeof _evt.clientY != 'undefined')
        {
          x = _evt.clientX;
          y = _evt.clientY;
        }
        else
        {
          x = _evt.pageX;
          y = _evt.pageY;
        }
      }
      return {x: x, y: y}
    }

    return me;
  })();

})( jQuery );